# frozen_string_literal: true

require 'dry-struct'
require 'i18n'
I18n.load_path += Dir[File.expand_path("../../config/locales", __dir__) + "/**/*.yml"]
I18n.default_locale = :nl
I18n.config.enforce_available_locales = true

require 'active_support/all'
require 'tempfile'

module Quby
  module Types
    include Dry.Types()
  end
end

require 'quby/array_attribute_valid_validator'
require 'quby/attribute_valid_validator'
require 'quby/markdown_parser'
require 'quby/range_categories'
require 'quby/type_validator'

require 'quby/compiler/instance'
require 'quby/compiler/entities'
require 'quby/compiler/dsl'
require 'quby/compiler/output'
require 'quby/compiler/outputs'

module Quby
  module Compiler
    def self.compile(key, sourcecode, path: nil, seeds:, lookup_tables:, last_update: nil, &block)
      Quby::Compiler::Instance.new(lookup_tables: lookup_tables).compile(
        key: key,
        sourcecode: sourcecode,
        path: path,
        seeds: seeds,
        last_update: last_update,
        &block
      )
    end

    def self.validate(key, sourcecode)
      Quby::Compiler::Instance.new(lookup_tables: lookup_tables).validate(
        key: key,
        sourcecode: sourcecode,
      )
    end
  end
end
