module Quby
  module Compiler
    module Entities
      # ScoreSchema instances describe score definitions.
      #
      # Score definitions are blocks of ruby code that return a hash of score results based on a questionnaire
      # response (answer). These schemas describe the purpose and form of the scores. Each key-value pair of the result
      # hash is called a subscore.
      # The :value subscore is treated as the main score result. Subscores are usually identified by their 'export_key'.
      # The score value's export_key is usually set to a shortened version of the main score key.
      class ScoreSchema < Dry::Struct
        # The key of the corresponding score in the questionnaire definition
        attribute :key, Types::Symbol
        # A label describing the general purpose of the score
        attribute :label, Types::String
        # An array of SubscoreSchemas describing each key that can be returned in the result hash of a score.
        attribute :subscore_schemas, Types::Array(Entities::SubscoreSchema)

        def export_key_labels
          subscore_schemas.map { |schema| [schema.export_key, schema.label] }.to_h.with_indifferent_access
        end
      end
    end
  end
end
