module Quby
  module Compiler
    module Entities
      # SubscoreSchema instances describe each key that could be returned in a score result hash
      class SubscoreSchema < Dry::Struct
        # The key this subscore has in the hash returned by the score
        attribute :key, Types::Symbol
        # The description of this subscore in the context of the score, like 'Mean', 'T-Score' or 'Interpretation'
        attribute :label, Types::String
        # The shortened key that will used as the field/column name for csv and oru exports,
        # excluding the questionnaire key part
        attribute :export_key, Types::Symbol
        # [Optional] Whether this score will only be exported through oru/api/data exports, but not shown in interfaces
        attribute :only_for_export?, Types::Bool
        # [Optional] The key of the variable definition used to calculate the subscore result.
        attribute :calculation_key?, Types::Symbol
        # [Optional argument] The name of the outcome table where this subscore should be shown. Used for cases where scores
        # differ in subscores too much to be shown as one table. By default, all scores end up in the `:main` table.
        # When outcome_table is explicitly nil, the value should not be shown in outcome tables
        attribute :outcome_table, Types::Symbol.optional.default(:main)
      end
    end
  end
end
