# frozen_string_literal: true

require 'redcarpet'

require 'quby/compiler/services/text_transformation'

module Quby
  class MarkdownParser
    include Quby::Compiler::Services::TextTransformation

    EXTENTIONS = {
      no_intra_emphasis: true
    }.freeze

    def initialize(source)
      @source = source || ""
    end

    def to_html
      rc_html = parser.render(@source).strip
      transform_special_text(rc_html)
    end

    def html_safe
      to_html.html_safe
    end

    private

    def parser
      @@parser ||= Redcarpet::Markdown.new(HTMLWithPants.new, EXTENTIONS)
    end
  end

  class HTMLWithPants < Redcarpet::Render::HTML
    include Redcarpet::Render::SmartyPants
  end
end
